@@ -96,7 +96,16 @@ remove_invalid <- function(gtfs_data, only_essential = TRUE, prompt_invalid = FA
       gtfs_data$trips    <- subset(gtfs_data$trips,    service_id %chin% service_ids)
       gtfs_data$calendar <- subset(gtfs_data$calendar, service_id %chin% service_ids)
     }
-  
+
+    # trips-calendar_dates relation (service_id)
+    if(!only_essential & !is.null(gtfs_data$calendar_dates)){
+      service_ids <- intersect(gtfs_data$trips$service_id, gtfs_data$calendar_dates$service_id)
+      removed$service_ids <- c(removed$service_ids, setdiff(gtfs_data$trips$service_id, gtfs_data$calendar_dates$service_id))
+      
+      gtfs_data$trips          <- subset(gtfs_data$trips,          service_id %chin% service_ids)
+      gtfs_data$calendar_dates <- subset(gtfs_data$calendar_dates, service_id %chin% service_ids)
+    }
+
     newsize <- object.size(gtfs_data)
   }
 
@@ -218,10 +227,51 @@ filter_valid_stop_times <- function(gtfs_data){
   return(gtfs_data)
 }
 
+#' @title Filter GTFS trips operating on given days
+#' 
+#' @description Filter a GTFS data read using gtfs2gps::read_gtfs(). It removes the
+#' trips operating in non-selected days. Note that it might produce
+#' inconsistent outputs that can be removed by using gtfs2gps::remove_invalid().
+#' @param gtfs_data A list of data.tables read using gtfs2gps::reag_gtfs().
+#' @param days A vector of selected week days written in the same way of column names
+#' in calendar file of GTFS (sunday, monday, etc.).
+#' @return A filtered GTFS data with the trips only from the selected days. 
+#' @export
+#' @examples
+#' poa <- read_gtfs(system.file("extdata/poa.zip", package = "gtfs2gps"))
+#' 
+#' subset <- filter_by_day(poa, c("wednesday", "friday"))
+filter_by_day <- function(gtfs_data, days){
+  if(!is.null(gtfs_data$calendar_dates)){
+    # do not use data.table operators to avoid changing the argument itself
+    gtfs_data$calendar_dates$mdate <- as.Date(paste(gtfs_data$calendar_dates$date), format = "%Y%m%d")
+
+    w_days <- c("sunday", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday")
+    gtfs_data$calendar_dates$mdate <- w_days[as.POSIXlt(gtfs_data$calendar_dates$mdate)$wday + 1]
+    
+    gtfs_data$calendar_dates <- subset(gtfs_data$calendar_dates, mdate %in% days)
+    serviceids <- unique(gtfs_data$calendar_dates$service_id)
+    gtfs_data$trips <- subset(gtfs_data$trips, service_id %in% serviceids)
+    gtfs_data$calendar_dates$mdate <- NULL
+  }
+  else if(!is.null(gtfs_data$calendar)){
+    code <- parse(text = paste(days, "> 0", collapse = " | "))
+    calendar_temp <- subset(gtfs_data$calendar, eval(code))
+    serviceids <- unique(calendar_temp$service_id)
+    gtfs_data$trips <- subset(gtfs_data$trips, service_id %in% serviceids)
+    
+    gtfs_data$calendar[, (days) := 0]
+  }
+  else stop("GTFS data does not have calendar_dates nor calendar")
+
+  return(gtfs_data)
+}
+
 #' @title Filter GTFS trips operating on week days
 #' 
 #' @description Filter a GTFS data read using gtfs2gps::read_gtfs(). It removes the
-#' trips operating only saturday or sunday.
+#' trips operating only on Saturdays or Sundays. Note that it might produce
+#' inconsistent outputs that can be removed by using gtfs2gps::remove_invalid().
 #' @param gtfs_data A list of data.tables read using gtfs2gps::reag_gtfs().
 #' @return A filtered GTFS data. 
 #' @export
@@ -308,9 +358,8 @@ filter_by_route_type <- function(gtfs_data, route_types) {
 #' @title Filter GTFS data by route ids
 #' 
 #' @description Filter a GTFS data by its route ids, subsetting routes
-#' and trips. It also removes the
-#' unnecessary stop_times, shapes, frequencies (if exist in a feed), 
-#' and stops accordingly.
+#' and trips. It also removes the unnecessary stop_times, shapes, frequencies
+#' (if exist in a feed), and stops accordingly.
 #' @param gtfs_data A list of data.tables read using gtfs2gps::reag_gtfs().
 #' @param route_ids A vector of route ids belonging to the routes of the
 #' gtfs_data data. Note that route_id might be loaded by gtfs2gps::read_gtfs()
@@ -341,3 +390,44 @@ filter_by_route_id <- function(gtfs_data, route_ids) {
   
   return(gtfs_data)
 }
+
+
+
+#' @title Remove GTFS data by route ids
+#' 
+#' @description Remove a GTFS data by its route ids, dropping routes
+#' and trips. It also removes the unnecessary stop_times, shapes, frequencies
+#' (if exist in a feed), and stops accordingly.
+#' @param gtfs_data A list of data.tables read using gtfs2gps::reag_gtfs().
+#' @param route_ids A vector of route ids belonging to the routes of the
+#' gtfs_data data. Note that route_id might be loaded by gtfs2gps::read_gtfs()
+#' as a string or a number, depending on the available values.
+#' @return A filtered GTFS data without service information of the set route ids. 
+#' @export
+#' @examples
+#' warsaw <- read_gtfs(system.file("extdata/warsaw.zip", package="gtfs2gps"))
+#' 
+#' subset <- remove_by_route_id(warsaw, c("15", "175"))
+remove_by_route_id <- function(gtfs_data, route_ids) {
+  if(is.null(gtfs_data$routes)) stop("GTFS data does not have routes")
+  
+  `%nin%` = Negate(`%in%`)
+  `%nchin%` = Negate(`%chin%`)
+  
+  gtfs_data$routes <- subset(gtfs_data$routes, route_id %nin% route_ids)
+  gtfs_data$trips <- subset(gtfs_data$trips, route_id %nin% route_ids) 
+  
+  shape_ids <- unique(gtfs_data$trips$shape_id)
+  gtfs_data$shapes <- subset(gtfs_data$shapes, shape_id %in% shape_ids)
+  
+  trip_ids <- unique(gtfs_data$trips$trip_id)
+  gtfs_data$stop_times <- subset(gtfs_data$stop_times, trip_id %chin% trip_ids)
+  
+  if(!is.null(gtfs_data$frequencies))
+    gtfs_data$frequencies <- subset(gtfs_data$frequencies, trip_id %chin% trip_ids)
+  
+  stop_ids <- unique(gtfs_data$stop_times$stop_id)
+  gtfs_data$stops <- subset(gtfs_data$stops, stop_id %in% stop_ids)
+  
+  return(gtfs_data)
+}
