@@ -19,14 +19,8 @@ write_gtfs <- function(gtfs, zipfile){
   tempd <- file.path(tempdir(), "gtfsdir") # create temp dir to save GTFS unzipped files
   unlink(normalizePath(paste0(tempd, "/", dir(tempd)), mustWork = FALSE), recursive = TRUE) # clean tempfiles in that dir
 
-  if(!is.null(gtfs$agency))      data.table::fwrite(gtfs$agency,      paste0(tempd, "/agency.txt"))
-  if(!is.null(gtfs$routes))      data.table::fwrite(gtfs$routes,      paste0(tempd, "/routes.txt"))
-  if(!is.null(gtfs$stops))       data.table::fwrite(gtfs$stops,       paste0(tempd, "/stops.txt"))
-  if(!is.null(gtfs$stop_times))  data.table::fwrite(gtfs$stop_times,  paste0(tempd, "/stop_times.txt"))
-  if(!is.null(gtfs$shapes))      data.table::fwrite(gtfs$shapes,      paste0(tempd, "/shapes.txt"))
-  if(!is.null(gtfs$trips))       data.table::fwrite(gtfs$trips,       paste0(tempd, "/trips.txt"))
-  if(!is.null(gtfs$calendar))    data.table::fwrite(gtfs$calendar,    paste0(tempd, "/calendar.txt"))
-  if(!is.null(gtfs$frequencies)) data.table::fwrite(gtfs$frequencies, paste0(tempd, "/frequencies.txt"))
+  for(attr in names(gtfs))
+    data.table::fwrite(gtfs[[attr]], paste0(tempd, "/", attr, ".txt"))
 
   # utils::zip(zipfile = zipfile, files = list.files(tempd, full.names = TRUE), flags = "-jr9Xq")
   zip::zipr(zipfile = zipfile, files = list.files(tempd, full.names = TRUE))
