@@ -13,13 +13,11 @@ test_that("snap_points", {
   shape_points <- sf::st_as_sf(shape_points, coords = c('X', 'Y'), agr = "identity", crs = 31982)
 
   res <- 15
-  result <- cpp_snap_points(stops %>% sf::st_coordinates(), stops %>% sf::st_coordinates(), res, "abc")
+  result <- cpp_snap_points(stops %>% sf::st_coordinates(), stops %>% sf::st_coordinates(), res)
   
   expect_equal(result, 1)
 
-  expect_warning(cpp_snap_points(stops %>% sf::st_coordinates(), matrix(2, 1), res, "abc"),
-               "Could not find a nearest point closer than 120m (eight times spatial_resolution) for stop 'abc'. Ignoring it.",
-               fixed = TRUE)
+  expect_length(cpp_snap_points(stops %>% sf::st_coordinates(), matrix(2, 1), res), 0)
 
   # complete test
   gtfs <- read_gtfs(system.file("extdata/poa.zip", package="gtfs2gps")) %>%
@@ -33,8 +31,8 @@ test_that("snap_points", {
   shape_points <- sf::st_as_sf(shape_points, coords = c('X', 'Y'), agr="identity", crs = 31982)
   
   res <- 3000
-  result <- cpp_snap_points(stops %>% sf::st_coordinates(), shape_points %>% sf::st_coordinates(), res, "abc")
+  result <- cpp_snap_points(stops %>% sf::st_coordinates(), shape_points %>% sf::st_coordinates(), res)
   
-  expect_equal(length(result), 62)
+  expect_equal(length(result), 0)
   expect_true(all(result == sort(result)))
 })
